@@ -22,7 +22,6 @@ import (
 	"errors"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"os"
 	"path"
 	"path/filepath"
@@ -34,6 +33,7 @@ import (
 	dbus "github.com/godbus/dbus/v5"
 	"github.com/linuxdeepin/dde-api/lang_info"
 	"github.com/linuxdeepin/dde-daemon/accounts1/users"
+	"github.com/linuxdeepin/dde-daemon/common/sessionmsg"
 	"github.com/linuxdeepin/go-lib/dbusutil"
 	"github.com/linuxdeepin/go-lib/gdkpixbuf"
 	"github.com/linuxdeepin/go-lib/imgutil"
@@ -222,9 +222,14 @@ func (u *User) SetMaxPasswordAge(sender dbus.Sender, nDays int32) *dbus.Error {
 	return nil
 }
 
-func (u *User) IsPasswordExpired() (expired bool, busErr *dbus.Error) {
-	expired, err := users.IsPasswordExpired(u.UserName)
-	return expired, dbusutil.ToError(err)
+func (u *User) IsPasswordExpired() (bool, *dbus.Error) {
+	// LDAP 域用户密码由域服务器控制，系统不去做密码过期检测
+	if users.IsLDAPDomainUserID(u.Uid) {
+		return false, nil
+	}
+
+	v, err := users.IsPasswordExpired(u.UserName)
+	return v, dbusutil.ToError(err)
 }
 
 func (u *User) SetLocked(sender dbus.Sender, locked bool) *dbus.Error {
@@ -314,34 +319,21 @@ func (u *User) SetAutomaticLogin(sender dbus.Sender, enabled bool) *dbus.Error {
 		return dbusutil.ToError(err)
 	}
 
-	u.PropsMu.Lock()
-	defer u.PropsMu.Unlock()
+	return u.setAutomaticLogin(enabled)
+}
 
-	if u.Locked {
-		return dbusutil.ToError(fmt.Errorf("user %s has been locked", u.UserName))
-	}
+// 设置用户是否快速登录
+func (u *User) SetQuickLogin(sender dbus.Sender, enabled bool) *dbus.Error {
+	logger.Infof("DBus call SetQuickLogin sender %v, enabled %t", sender, enabled)
 
-	if u.AutomaticLogin == enabled {
-		return nil
-	}
-
-	var name = u.UserName
-	if !enabled {
-		name = ""
-	}
-
-	session := u.XSession
-	if session == "" {
-		session = getUserSession(u.HomeDir)
-	}
-	if err := users.SetAutoLoginUser(name, session); err != nil {
-		logger.Warning("DoAction: set auto login failed:", err)
+	// 验证调用者权限
+	err := u.checkAuthQuickLogin(sender, enabled)
+	if err != nil {
+		logger.Warning(err)
 		return dbusutil.ToError(err)
 	}
 
-	u.AutomaticLogin = enabled
-	_ = u.emitPropChangedAutomaticLogin(enabled)
-	return nil
+	return u.setQuickLogin(enabled)
 }
 
 func (u *User) EnableNoPasswdLogin(sender dbus.Sender, enabled bool) *dbus.Error {
@@ -420,7 +412,7 @@ func (u *User) SetLocale(sender dbus.Sender, locale string) *dbus.Error {
 func (u *User) SetLayout(sender dbus.Sender, layout string) *dbus.Error {
 	logger.Debug("[SetLayout] new layout:", layout)
 
-	err := u.checkAuth(sender, true, polkitActionSetKeyboardLayout)
+	err := u.checkAuth(sender, true, "")
 	if err != nil {
 		logger.Debug("[SetLayout] access denied:", err)
 		return dbusutil.ToError(err)
@@ -531,6 +523,23 @@ func (u *User) SetIconFile(sender dbus.Sender, iconURI string) *dbus.Error {
 			}
 		}
 
+		// 默认的icon如果不是default.png的情况下，会被设置为customIcon,会被误删,
+		// 为规避已经设置为customIcon的历史版本,判断如果是icons目录下的数据,就不删除
+		needRemove := func(uri string) bool {
+			if uri == "" {
+				return true
+			}
+			fPath := dutils.DecodeURI(u.customIcon)
+			return filepath.Dir(fPath) != "/var/lib/AccountsService/icons"
+		}
+		// remove old custom icon
+		if needRemove(u.customIcon) {
+			logger.Debugf("remove old custom icon %q", u.customIcon)
+			err := os.Remove(dutils.DecodeURI(u.customIcon))
+			if err != nil {
+				logger.Warning(err)
+			}
+		}
 		u.customIcon = newIconURI
 		u.customIconList = append(u.customIconList, newIconURI)
 		u.updateIconList()
@@ -591,6 +600,62 @@ func (u *User) DeleteIconFile(sender dbus.Sender, icon string) *dbus.Error {
 	return nil
 }
 
+func (u *User) EnableWechatAuth(sender dbus.Sender, value bool) *dbus.Error {
+	logger.Infof("DBus call EnableWechatAuth sender %v, enabled %t", sender, value)
+	// 验证调用者权限
+	if !u.checkIsControlCenter(sender) {
+		return dbusutil.ToError(fmt.Errorf("not allow %v call this method", sender))
+	}
+	return u.enableWechatAuth(value)
+}
+func (u *User) UpdateWechatAuthState() *dbus.Error {
+	logger.Infof("DBus call UpdateWechatAuthState")
+	// 通过synchelper检查本地账户是否有绑定的UOS ID
+	syncObj := u.service.Conn().Object("com.deepin.sync.Helper", "/com/deepin/sync/Helper")
+	var uosid string
+	err := syncObj.Call("com.deepin.sync.Helper.UOSID", 0).Store(&uosid)
+	if err != nil {
+		logger.Warning(err)
+		return dbusutil.ToError(err)
+	}
+	var ubid string
+	err = syncObj.Call("com.deepin.sync.Helper.LocalBindCheck", 0, uosid, u.UUID).Store(&ubid)
+	if err != nil {
+		logger.Warning(err)
+		return dbusutil.ToError(err)
+	}
+	var isBindWechat bool
+	var wechatNickName string
+	err = syncObj.Call("com.deepin.sync.Helper.LocalBindCheckWechat", 0, uosid, u.UUID).Store(&isBindWechat, &wechatNickName)
+	if err != nil {
+		logger.Warning(err)
+		return dbusutil.ToError(err)
+	}
+	logger.Infof("DBus call UpdateWechatAuthState ubid %v, wechatNickName %v", ubid, wechatNickName)
+	if (ubid == "" || !isBindWechat) && u.WechatAuthEnabled {
+		return u.enableWechatAuth(false)
+	}
+
+	return nil
+}
+
+func (u *User) enableWechatAuth(value bool) *dbus.Error {
+	u.PropsMu.Lock()
+	defer u.PropsMu.Unlock()
+
+	if value == u.WechatAuthEnabled {
+		return nil
+	}
+
+	err := u.writeUserConfigWithChange(confKeyWechatAuthEnabled, value)
+	if err != nil {
+		return dbusutil.ToError(err)
+	}
+
+	u.setPropWechatAuthEnabled(value)
+	return nil
+}
+
 func (u *User) SetDesktopBackgrounds(sender dbus.Sender, val []string) *dbus.Error {
 	logger.Debugf("[SetDesktopBackgrounds] val: %#v", val)
 
@@ -659,7 +724,17 @@ func (u *User) SetCurrentWorkspace(sender dbus.Sender, currentWorkspace int32) *
 
 func (u *User) SetGreeterBackground(sender dbus.Sender, bg string) *dbus.Error {
 	logger.Debug("[SetGreeterBackground] new background:", bg)
-
+	if dutils.IsFileExist("/var/lib/deepin/permission-manager/wallpaper_locked") {
+		_ = sessionmsg.SendMessage(sessionmsg.NewMessage(true, &sessionmsg.BodyNotify{
+			Icon: "preferences-system",
+			Body: &sessionmsg.LocalizeStr{
+				Format: Tr("This system wallpaper is locked. Please contact your admin."),
+				Args:   []string{},
+			},
+			ExpireTimeout: -1,
+		}))
+		return dbusutil.ToError(errors.New("do not have permission to change greeter background"))
+	}
 	err := u.checkAuth(sender, true, "")
 	if err != nil {
 		logger.Debug("[SetGreeterBackground] access denied:", err)
@@ -1020,14 +1095,15 @@ func (u *User) PasswordExpiredInfo() (expiredStatus ExpiredStatus, dayLeft int64
 
 	// pam_unix/passverify.c
 	curDays := time.Now().Unix() / secondsPerDay
-	daysLeft := spLastChg + spMax - curDays + 1
+	daysLeft := spLastChg + spMax - curDays
 
-	if daysLeft <= 0 {
+	// daysLeft < 0时为已经过期，等于0时即当天过期时按1天算，因此大于等于0时需要加1天
+	if daysLeft < 0 {
 		return expiredStatusExpiredAlready, daysLeft, nil
-	} else if spWarn >= daysLeft {
-		return expiredStatusExpiredSoon, daysLeft, nil
+	} else if spWarn > daysLeft {
+		return expiredStatusExpiredSoon, daysLeft + 1, nil
 	}
-	return expiredStatusNormal, daysLeft, nil
+	return expiredStatusNormal, daysLeft + 1, nil
 }
 
 func (u *User) SetPasswordHint(hint string) (busErr *dbus.Error) {
@@ -1044,7 +1120,8 @@ func (u *User) GetReminderInfo() (info LoginReminderInfo, dbusErr *dbus.Error) {
 }
 
 /* secret question */
-const secretQuestionDirectory = "/var/lib/dde-daemon/secret-question/" //#nosec G101
+// #nosec G101
+const secretQuestionDirectory = "/var/lib/dde-daemon/secret-question/"
 
 func walkQuestions(username string, f func(id int, salt string) error) error {
 	path := filepath.Join(secretQuestionDirectory, username)
@@ -1171,7 +1248,7 @@ func (u *User) SetSecretQuestions(sender dbus.Sender, list map[int][]byte) *dbus
 		content.WriteRune('\n')
 	}
 
-	err = ioutil.WriteFile(path, content.Bytes(), 0600)
+	err = os.WriteFile(path, content.Bytes(), 0600)
 	return dbusutil.ToError(err)
 }
 
@@ -1224,3 +1301,12 @@ func (u *User) DeleteSecretKey(sender dbus.Sender) *dbus.Error {
 	}
 	return nil
 }
+
+func (u *User) deleteSecretKey() error {
+	if u.uadpInterface == nil {
+		logger.Warning("uadpInterface is nil")
+		return nil
+	}
+
+	return u.uadpInterface.Delete(0, u.UserName)
+}
