@@ -156,13 +156,7 @@ func (a *Audio) needAutoSwitchInputPort() bool {
 		return false
 	}
 
-	// 同一级别的不切换
-	if inputs.GetPortType(currentCardName, currentPortName) == firstPort.PortType {
-		logger.Debugf("current input<%s,%s> has the same type with first port", currentCardName, currentPortName)
-		return false
-	}
-
-	logger.Infof("will auto switch from input<%s,%s> to input<%s,%s>",
+	logger.Debugf("will auto switch from input<%s,%s> to input<%s,%s>",
 		currentCardName, currentPortName, firstPort.CardName, firstPort.PortName)
 	return true
 }
@@ -173,8 +167,7 @@ func (a *Audio) needAutoSwitchOutputPort() bool {
 		return false
 	}
 
-	outputs := GetPriorityManager().Output
-	firstPort := outputs.GetTheFirstPort()
+	firstPort := a.tryGetPreferOutPut()
 
 	// 没有可用端口
 	if firstPort.PortType == PortTypeInvalid {
@@ -204,25 +197,19 @@ func (a *Audio) needAutoSwitchOutputPort() bool {
 		return false
 	}
 
-	// 同一级别的不切换
-	if outputs.GetPortType(currentCardName, currentPortName) == firstPort.PortType {
-		logger.Debugf("current input<%s,%s> has the same type with first port", currentCardName, currentPortName)
-		return false
-	}
-
-	logger.Infof("will auto switch from output<%s,%s> to output<%s,%s>",
+	logger.Debugf("will auto switch from output<%s,%s> to output<%s,%s>",
 		currentCardName, currentPortName, firstPort.CardName, firstPort.PortName)
 	return true
 }
 
 func (a *Audio) autoSwitchPort() {
+	logger.Warning("auto switch port")
 	if a.needAutoSwitchOutputPort() {
-		outputs := GetPriorityManager().Output
-		firstOutput := outputs.GetTheFirstPort()
+		firstOutput := a.tryGetPreferOutPut()
 		card, err := a.cards.getByName(firstOutput.CardName)
 
 		if err == nil {
-			logger.Infof("auto switch output to #%d %s:%s", card.Id, card.core.Name, firstOutput.PortName)
+			logger.Warningf("auto switch output to #%d %s:%s", card.Id, card.core.Name, firstOutput.PortName)
 			a.setPort(card.Id, firstOutput.PortName, pulse.DirectionSink)
 		} else {
 			logger.Warning(err)
@@ -239,12 +226,13 @@ func (a *Audio) autoSwitchPort() {
 	}
 
 	if a.needAutoSwitchInputPort() {
+		logger.Warning("auto switch input")
 		inputs := GetPriorityManager().Input
 		firstInput := inputs.GetTheFirstPort()
 		card, err := a.cards.getByName(firstInput.CardName)
 
 		if err == nil {
-			logger.Infof("auto switch input to #%d %s:%s", card.Id, card.core.Name, firstInput.PortName)
+			logger.Debugf("auto switch input to #%d %s:%s", card.Id, card.core.Name, firstInput.PortName)
 			a.setPort(card.Id, firstInput.PortName, pulse.DirectionSource)
 		} else {
 			logger.Warning(err)
@@ -298,16 +286,6 @@ func (a *Audio) handleCardEvent(eventType int, idx uint32) {
 func (a *Audio) handleCardAdded(idx uint32) {
 	// 数据更新在refreshCards中统一处理，这里只做业务逻辑上的响应
 	logger.Debugf("card %d added", idx)
-
-	card, err := a.cards.get(idx)
-	if err != nil {
-		logger.Warningf("invalid card index #%d", idx)
-		return
-	}
-
-	if isBluezAudio(card.core.Name) {
-		card.AutoSetBluezMode()
-	}
 }
 
 func (a *Audio) handleCardRemoved(idx uint32) {
@@ -327,12 +305,8 @@ func (a *Audio) handleCardChanged(idx uint32) {
 	}
 
 	// 如果发生变化的是当前输出所用的声卡，且是蓝牙声卡
-	if a.defaultSink != nil && idx == a.defaultSink.Card && isBluetoothCard(card.core) {
-		if strings.Contains(strings.ToLower(card.ActiveProfile.Name), bluezModeA2dp) {
-			a.setPropBluetoothAudioMode(bluezModeA2dp)
-		} else if strings.Contains(strings.ToLower(card.ActiveProfile.Name), bluezModeHeadset) {
-			a.setPropBluetoothAudioMode(bluezModeHeadset)
-		}
+	if a.defaultSink != nil && idx == a.defaultSink.Card && isBluezAudio(card.core.Name) {
+		a.setPropBluetoothAudioMode(card.ActiveProfile.Name)
 	}
 }
 
@@ -370,10 +344,10 @@ func (a *Audio) handleSinkChanged(idx uint32) {
 	logger.Debugf("sink %d changed", idx)
 
 	// 蓝牙模式切换、触发sink change
-	if a.defaultSink.index == idx && isBluezAudio(a.defaultSink.Name) {
+	if a.defaultSink != nil && a.defaultSink.index == idx && isBluezAudio(a.defaultSink.Name) {
 		card, err := a.cards.get(a.defaultSink.Card)
 		if err == nil {
-			a.setPropBluetoothAudioMode(card.BluezMode())
+			a.setPropBluetoothAudioMode(card.ActiveProfile.Name)
 		} else {
 			logger.Warningf("%d card not found", a.defaultSink.Card)
 		}
@@ -581,6 +555,15 @@ func (a *Audio) listenGSettingVolumeIncreaseChanged() {
 	})
 }
 
+func (a *Audio) listenGSettingDeviceManageChanged() {
+	gsettings.ConnectChanged(gsSchemaControlCenter, gsKeyDeviceManager, func(val string) {
+		if a.enableAutoSwitchPort != a.controlCenterDeviceManager.Get() {
+			a.controlCenterDeviceManager.Set(a.enableAutoSwitchPort)
+			logger.Info("listenGSettingDeviceManageChanged set a.enableAutoSwitchPort to a.controlCenterDeviceManager. value : ", a.enableAutoSwitchPort)
+		}
+	})
+}
+
 // 外部修改ReducecNoise时触发回调，响应实际降噪开关
 func (a *Audio) writeReduceNoise(write *dbusutil.PropertyWrite) *dbus.Error {
 	reduce, ok := write.Value.(bool)
