@@ -8,12 +8,7 @@ import (
 	"syscall"
 
 	"github.com/godbus/dbus/v5"
-	login1 "github.com/linuxdeepin/go-dbus-factory/system/org.freedesktop.login1"
-)
-
-const (
-	dbusInhibitorPath      = "/org/deepin/InhibitHint1"
-	dbusInhibitorInterface = "org.deepin.InhibitHint1"
+	"github.com/linuxdeepin/go-dbus-factory/system/org.freedesktop.login1"
 )
 
 func (m *Grub2) enableShutdown() {
@@ -25,6 +20,7 @@ func (m *Grub2) enableShutdown() {
 			logger.Infof("enable shutdown")
 		}
 		m.inhibitFd = -1
+		m.dbusObj.RemoveAllHandlers()
 	}
 }
 
@@ -39,6 +35,28 @@ func (m *Grub2) preventShutdown() {
 		}
 		m.inhibitFd = fd
 	}
+	// handle login1 restart
+	_, _ = m.dbusObj.ConnectNameOwnerChanged(func(name string, oldOwner string, newOwner string) {
+		if name == "org.freedesktop.login1" && newOwner != "" && oldOwner == "" {
+			if m.inhibitFd != -1 { // 如果之前存在inhibit时，login1重启需要重新inhibit
+				err := syscall.Close(int(m.inhibitFd))
+				m.inhibitFd = -1
+				if err != nil {
+					logger.Warning("failed to close fd:", err)
+					return
+				}
+				fd, err := inhibit("shutdown", dbusServiceName,
+					"Updating the system, please shut down or reboot later.")
+				logger.Infof("prevent shutdown: fd:%v\n", fd)
+				if err != nil {
+					logger.Warning(err)
+				} else {
+					m.inhibitFd = fd
+				}
+			}
+		}
+	})
+	// end handle login1 restart
 }
 
 func inhibit(what, who, why string) (dbus.UnixFD, error) {
