@@ -7,7 +7,6 @@ package power
 import (
 	"encoding/json"
 	"errors"
-	"fmt"
 	"math"
 	"os"
 	"strings"
@@ -18,6 +17,7 @@ import (
 	"github.com/linuxdeepin/go-lib/dbusutil/gsprop"
 
 	"github.com/godbus/dbus/v5"
+	ConfigManager "github.com/linuxdeepin/go-dbus-factory/org.desktopspec.ConfigManager"
 	"github.com/linuxdeepin/go-lib/dbusutil"
 	"github.com/linuxdeepin/go-lib/gsettings"
 	"github.com/linuxdeepin/go-lib/procfs"
@@ -28,11 +28,21 @@ import (
 
 const submodulePSP = "PowerSavePlan"
 
+const (
+	dsettingsAppID                                     = "org.deepin.dde.daemon"
+	dsettingsPowerName                                 = "org.deepin.dde.daemon.power"
+	dsettingsAllowScreenSaver                          = "allowScreenSaver"
+	dsettingsDelayWakeupInterval                       = "delayWakeupInterval"
+	dsettingsDelayHandleIdleOffIntervalWhenScreenBlack = "delayHandleIdleOffIntervalWhenScreenBlack"
+)
+
 func init() {
 	submoduleList = append(submoduleList, newPowerSavePlan)
 }
 
 type powerSavePlan struct {
+	systemSigLoop      *dbusutil.SignalLoop
+	dsgPower           ConfigManager.Manager
 	manager            *Manager
 	screenSaverTimeout int32
 	metaTasks          metaTasks
@@ -50,25 +60,36 @@ type powerSavePlan struct {
 	multiBrightnessWithPsm *multiBrightnessWithPsm
 	psmEnabledTime         time.Time
 	psmPercentChangedTime  time.Time
+	modeBeforeIdle         string
+	allowScreenSaver       bool
 }
 
 func newPowerSavePlan(manager *Manager) (string, submodule, error) {
 	p := new(powerSavePlan)
 	p.manager = manager
 
-	conn := manager.helper.xConn
 	var err error
-	p.atomNetWMStateFullscreen, err = conn.GetAtom("_NET_WM_STATE_FULLSCREEN")
-	if err != nil {
-		return submodulePSP, nil, err
-	}
-	p.atomNetWMStateFocused, err = conn.GetAtom("_NET_WM_STATE_FOCUSED")
-	if err != nil {
-		return submodulePSP, nil, err
+	if !manager.UseWayland {
+		conn := manager.helper.xConn
+		p.atomNetWMStateFullscreen, err = conn.GetAtom("_NET_WM_STATE_FULLSCREEN")
+		if err != nil {
+			return submodulePSP, nil, err
+		}
+		p.atomNetWMStateFocused, err = conn.GetAtom("_NET_WM_STATE_FOCUSED")
+		if err != nil {
+			return submodulePSP, nil, err
+		}
 	}
 
 	p.fullscreenWorkaroundAppList = manager.settings.GetStrv(
 		"fullscreen-workaround-app-list")
+
+	err = p.initDsgConfig()
+	if err != nil {
+		logger.Warning(err)
+	}
+	p.systemSigLoop.Start()
+
 	return submodulePSP, p, nil
 }
 
@@ -156,7 +177,7 @@ func (psp *powerSavePlan) Start() error {
 		psp.initMultiBrightnessWithPsm()
 	}
 
-	//OnBattery changed will effect current PowerSavePlan
+	// OnBattery changed will effect current PowerSavePlan
 	err := power.OnBattery().ConnectChanged(func(hasValue bool, value bool) {
 		psp.Reset()
 	})
@@ -224,24 +245,53 @@ func (psp *powerSavePlan) dealWithPowerSavingModeWhenSystemBoot() {
 	}
 }
 
+// 分级以前逻辑有问题，目前先按无分级处理，保留该判断方式后续观望
+func (psp *powerSavePlan) powerSavingModeIsMultiLevelAdjustment(maxBacklightBrightness float64) bool {
+	const (
+		multiLevelAdjustmentThreshold = 100 // 分级调节判断阈值，最大亮度值小于该值且不为0时，调节方式为分级调节
+	)
+	// 判断亮度调节方式是分级调节还是百分比滑动：最大亮度小于100且最大亮度不为0时，为分级调节
+	return maxBacklightBrightness < multiLevelAdjustmentThreshold && maxBacklightBrightness != 0
+}
+
+// 节能模式亮度统一策略处理：根据当前非节能模式亮度和自动降低亮度比例配置，计算出节能模式需要降低的亮度
+func (psp *powerSavePlan) powerSavingModeBrightnessDrop(brightness, scale float64) float64 {
+	newBrightness := math.Round(brightness * 100 * (1 - scale/100))
+	newBrightness = newBrightness / 100
+	if newBrightness > 1.0 {
+		newBrightness = 1.0
+	} else if newBrightness < 0.1 {
+		newBrightness = 0.1
+	}
+	return newBrightness
+}
+
+// 节能模式亮度统一策略处理：根据当前节能模式下的亮度和自动降低亮度比例配置，计算出恢复到非节能模式后的亮度
+func (psp *powerSavePlan) powerSavingModeBrightnessRestored(brightness, scale float64) float64 {
+	newBrightness := math.Round(brightness * 100 / (1 - scale/100))
+	newBrightness = newBrightness / 100
+	if newBrightness > 1.0 {
+		newBrightness = 1.0
+	} else if newBrightness < 0.1 {
+		newBrightness = 0.1
+	}
+	return newBrightness
+}
+
 // 节能模式降低亮度的比例,并降低亮度
 func (psp *powerSavePlan) handlePowerSavingModeBrightnessDropPercentChanged(hasValue bool, lowerValue uint32) {
 	if !hasValue {
 		return
 	}
 	logger.Debug("power saving mode lower brightness changed to", lowerValue)
-	newLowerBrightnessScale := float64(lowerValue)
 	psp.manager.PropsMu.RLock()
 	hasLightSensor := psp.manager.HasAmbientLightSensor
 	psp.manager.PropsMu.RUnlock()
-
 	if hasLightSensor && psp.manager.AmbientLightAdjustBrightness.Get() {
 		return
 	}
-	psp.manager.savingModeBrightnessDropPercent.Set(int32(lowerValue))
-	savingModeEnable, err := psp.manager.helper.Power.PowerSavingModeEnabled().Get(0)
-	if err != nil {
-		logger.Error("get current power savingMode state error : ", err)
+	if !psp.manager.isSessionActive() { // 系统级的调节保证只有激活用户才能做逻辑
+		return
 	}
 
 	brightnessTable, err := psp.manager.helper.Display.GetBrightness(0)
@@ -249,31 +299,23 @@ func (psp *powerSavePlan) handlePowerSavingModeBrightnessDropPercentChanged(hasV
 		logger.Warning(err)
 		return
 	}
-	if !psp.manager.isSessionActive() { // 系统级的调节保证只有激活用户才能做逻辑
-		return
+	oldLowerBrightnessScale := float64(psp.manager.savingModeBrightnessDropPercent.Get())
+	newLowerBrightnessScale := float64(lowerValue)
+	psp.manager.savingModeBrightnessDropPercent.Set(int32(lowerValue))
+	savingModeEnable, err := psp.manager.helper.Power.PowerSavingModeEnabled().Get(0)
+	if err != nil {
+		logger.Error("get current power savingMode state error : ", err)
 	}
 
 	if savingModeEnable {
-		// adjust brightness by lowerBrightnessScale
-		// 判断亮度修改是手动调节亮度还是调节了节能选项
-		lowerBrightnessScale := 1 - newLowerBrightnessScale/100
-		for key, value := range brightnessTable { // 反求未节能时的亮度
-			refBrightness, err := psp.multiBrightnessWithPsm.getReferenceBrightnessWhilePsmPercentChanged(key)
-			if err != nil {
-				logger.Warning(err)
-				continue
-			}
-
-			value = refBrightness * lowerBrightnessScale
-			if value < 0.1 {
-				value = 0.1
-			}
+		for key, value := range brightnessTable {
+			value = psp.powerSavingModeBrightnessRestored(value, oldLowerBrightnessScale)
+			value = psp.powerSavingModeBrightnessDrop(value, newLowerBrightnessScale)
 			brightnessTable[key] = value
-
 			psp.psmPercentChangedTime = time.Now()
 		}
 	} else {
-		//else中(非节能状态下的调节)不需要做响应,需要降低亮度的预设值在之前已经保存了
+		// else中(非节能状态下的调节)不需要做响应,需要降低亮度的预设值在之前已经保存了
 		return
 	}
 	psp.manager.setAndSaveDisplayBrightness(brightnessTable)
@@ -281,10 +323,6 @@ func (psp *powerSavePlan) handlePowerSavingModeBrightnessDropPercentChanged(hasV
 
 // 节能模式变化后的亮度修改
 func (psp *powerSavePlan) handlePowerSavingModeChanged(hasValue bool, enabled bool) {
-	const (
-		multiLevelAdjustmentScale     = 0.2 // 分级调节时，默认按照20%亮度调节值调整亮度，并在亮度显示的设置分级中，归到所在分级
-		multiLevelAdjustmentThreshold = 100 // 分级调节判断阈值，最大亮度值小于该值且不为0时，调节方式为分级调节
-	)
 	if !hasValue {
 		return
 	}
@@ -296,6 +334,15 @@ func (psp *powerSavePlan) handlePowerSavingModeChanged(hasValue bool, enabled bo
 		return
 	}
 
+	bInBootTime, err := psp.manager.helper.Power.IsInBootTime().Get(0)
+	if err != nil {
+		logger.Warning(err)
+	}
+	if bInBootTime {
+		logger.Debug("handlePowerSavingModeChanged InBootTime, Can't change brightness.")
+		return
+	}
+
 	psp.manager.PropsMu.RLock()
 	hasLightSensor := psp.manager.HasAmbientLightSensor
 	psp.manager.PropsMu.RUnlock()
@@ -309,48 +356,23 @@ func (psp *powerSavePlan) handlePowerSavingModeChanged(hasValue bool, enabled bo
 		logger.Warning(err)
 		return
 	}
-	maxBacklightBrightness, err := psp.manager.helper.Display.MaxBacklightBrightness().Get(0)
-	if err != nil {
-		logger.Warning(err)
-	}
-	// 判断亮度调节方式是分级调节还是百分比滑动：最大亮度小于100且最大亮度不为0时，为分级调节
-	isMultiLevelAdjustment := maxBacklightBrightness < multiLevelAdjustmentThreshold && maxBacklightBrightness != 0
-	// 判断亮度修改是手动调节亮度还是调节了节能选项
-	lowerBrightnessScale := 1 - float64(psp.manager.savingModeBrightnessDropPercent.Get())/100
-	oneStepValue := 1 / float64(maxBacklightBrightness)
-	numSteps := math.Round(float64(maxBacklightBrightness) * multiLevelAdjustmentScale)
-	if enabled {
-		// reduce brightness when enabled saveMode
-		for key, value := range brightnessTable {
-			if isMultiLevelAdjustment {
-				// 分级调节,减去需要降低的亮度
-				value -= oneStepValue * numSteps
-				if value < oneStepValue {
-					value = oneStepValue
-				}
-			} else {
-				// 非分级调节
-				value *= lowerBrightnessScale
-			}
-			if value < 0.1 {
-				value = 0.1
-			}
-			brightnessTable[key] = value
+	brightnessScale := float64(psp.manager.savingModeBrightnessDropPercent.Get())
+	for key, value := range brightnessTable {
+		if enabled {
+			value = psp.powerSavingModeBrightnessDrop(value, brightnessScale)
+		} else {
+			value = psp.powerSavingModeBrightnessRestored(value, brightnessScale)
 		}
+		brightnessTable[key] = value
+	}
 
+	if enabled {
 		psp.multiBrightnessWithPsm.init()
 		psp.setBrightnessFromDisplay()
 		psp.multiBrightnessWithPsm.mapToObject()
 		psp.setToBrightnessSave()
 		psp.psmEnabledTime = time.Now()
 	} else {
-		for _, val := range psp.multiBrightnessWithPsm.MultiBrightness {
-			if !val.ManuallyModified {
-				brightnessTable[val.MonitorName] = val.BrightnessSaved
-			} else {
-				brightnessTable[val.MonitorName] = val.BrightnessLatest
-			}
-		}
 		psp.brightnessSave.Set("")
 	}
 	psp.manager.setAndSaveDisplayBrightness(brightnessTable)
@@ -504,6 +526,11 @@ func (psp *powerSavePlan) startScreensaver() {
 		return
 	}
 
+	if !psp.allowScreenSaver {
+		logger.Info("do not start screensaver, allowScreenSaver is false")
+		return
+	}
+
 	startScreensaver()
 	psp.screensaverRunning = true
 }
@@ -519,9 +546,11 @@ func (psp *powerSavePlan) stopScreensaver() {
 func (psp *powerSavePlan) makeSystemSleep() {
 	logger.Info("sleep")
 	psp.stopScreensaver()
-	//psp.manager.setDPMSModeOn()
-	//psp.resetBrightness()
-	psp.manager.doSuspendByFront()
+	if psp.manager.UseWayland {
+		psp.manager.doSuspend()
+	} else {
+		psp.manager.doSuspendByFront()
+	}
 }
 
 func (psp *powerSavePlan) lock() {
@@ -559,6 +588,9 @@ func (psp *powerSavePlan) screenBlack() {
 	taskF := newDelayedTask("screenFullBlack", fullBlackTime, func() {
 		psp.stopScreensaver()
 		logger.Info("Screen full black")
+		if manager.ScreenBlackLock.Get() {
+			manager.lockWaitShow(5*time.Second, true)
+		}
 
 		if adjustBrightnessEnabled {
 			// set min brightness for all outputs
@@ -568,7 +600,7 @@ func (psp *powerSavePlan) screenBlack() {
 			}
 			manager.setDisplayBrightness(brightnessTable)
 		}
-		manager.doTurnOffScreen()
+		manager.setDPMSModeOff()
 
 	})
 	psp.addTask(taskF)
@@ -661,6 +693,7 @@ func (psp *powerSavePlan) HandleIdleOn() {
 		logger.Debugf("idle time: %d ms", idleTime)
 		psp.metaTasks.setRealDelay(idleTime)
 	}
+	psp.modeBeforeIdle, _ = psp.manager.helper.Power.Mode().Get(0)
 
 	for _, t := range psp.metaTasks {
 		logger.Debugf("do %s after %v", t.name, t.realDelay)
@@ -671,16 +704,29 @@ func (psp *powerSavePlan) HandleIdleOn() {
 	_, err := os.Stat("/etc/deepin/no_suspend")
 	if err == nil {
 		if psp.manager.ScreenBlackLock.Get() {
-			//m.setDPMSModeOn()
-			//m.lockWaitShow(4 * time.Second)
+			// m.setDPMSModeOn()
+			// m.lockWaitShow(4 * time.Second)
 			psp.manager.doLock(true)
 			time.Sleep(time.Millisecond * 500)
 		}
 	}
 }
 
-// 结束 Idle
-func (psp *powerSavePlan) HandleIdleOff() {
+func (ps *powerSavePlan) restoreDpmsStateFile() {
+	v, err := os.ReadFile("/tmp/dpms-state")
+	if err != nil {
+		return
+	}
+
+	if string(v) == "1" {
+		err = os.WriteFile("/tmp/dpms-state", []byte("0"), 0644)
+		if err != nil {
+			logger.Warning("WriteFile /tmp/dpms-state:", err)
+		}
+	}
+}
+
+func (psp *powerSavePlan) handleIdleOff() {
 	psp.mu.Lock()
 	defer psp.mu.Unlock()
 
@@ -690,12 +736,46 @@ func (psp *powerSavePlan) HandleIdleOff() {
 		return
 	}
 
+	if psp.modeBeforeIdle != "" && psp.modeBeforeIdle != "performance" {
+		err := psp.manager.helper.Power.SetMode(0, psp.modeBeforeIdle)
+		if err != nil {
+			logger.Warning(err)
+		}
+		psp.modeBeforeIdle = ""
+	}
+
 	psp.manager.setPrepareSuspend(suspendStateFinish)
 	logger.Info("HandleIdleOff")
 	psp.interruptTasks()
 	psp.manager.setDPMSModeOn()
-	psp.manager.setWmBlackScreenActive(false)
+	psp.manager.setDDEBlackScreenActive(false)
 	psp.resetBrightness()
+	psp.restoreDpmsStateFile()
+}
+
+// 结束 Idle
+func (psp *powerSavePlan) HandleIdleOff() {
+	var powerPressAction int32
+	if psp.manager.OnBattery {
+		powerPressAction = psp.manager.BatteryPressPowerBtnAction.Get()
+	} else {
+		powerPressAction = psp.manager.LinePowerPressPowerBtnAction.Get()
+	}
+
+	if powerPressAction == powerActionTurnOffScreen {
+		var delayHandleIdleOffInterval uint32
+		delayHandleIdleOffInterval = psp.manager.delayHandleIdleOffIntervalWhenScreenBlack
+		if delayHandleIdleOffInterval > 2500 {
+			// 当“按电源按钮时-关闭显示器”时，最长可增加2.5S延时
+			delayHandleIdleOffInterval = 2500
+		}
+		time.AfterFunc(time.Duration(delayHandleIdleOffInterval)*time.Millisecond, func() {
+			psp.handleIdleOff()
+		})
+		return
+	}
+
+	psp.handleIdleOff()
 }
 
 func (psp *powerSavePlan) isWindowFullScreenAndFocused(xid x.Window) (bool, error) {
@@ -709,10 +789,10 @@ func (psp *powerSavePlan) isWindowFullScreenAndFocused(xid x.Window) (bool, erro
 		if s == psp.atomNetWMStateFullscreen {
 			found++
 		}
-		//后端的代码之前是基于deepin-wm这个窗口适配，现在换成了kwin,state中没有 focus 这个属性了
-		//else if s == psp.atomNetWMStateFocused {
+		// 后端的代码之前是基于deepin-wm这个窗口适配，现在换成了kwin,state中没有 focus 这个属性了
+		// else if s == psp.atomNetWMStateFocused {
 		//	found++
-		//}
+		// }
 		if found == 1 {
 			return true, nil
 		}
@@ -811,8 +891,8 @@ func (psp *powerSavePlan) handleBrightnessPropertyChanged(value bool, value2 map
 
 type brightnessWithPsp struct {
 	MonitorName      string
-	BrightnessSaved  float64 //开启节能模式之前的亮度值
-	BrightnessLatest float64 //开启节能模式之后每次手动设置的亮度
+	BrightnessSaved  float64 // 开启节能模式之前的亮度值
+	BrightnessLatest float64 // 开启节能模式之后每次手动设置的亮度
 	ManuallyModified bool
 }
 
@@ -880,19 +960,6 @@ func (mb *multiBrightnessWithPsm) mapToObject() {
 	}
 }
 
-func (mb *multiBrightnessWithPsm) getReferenceBrightnessWhilePsmPercentChanged(key string) (float64, error) {
-	for _, v := range mb.MultiBrightness {
-		if v.MonitorName == key {
-			if v.ManuallyModified {
-				return v.BrightnessLatest, nil
-			} else {
-				return v.BrightnessSaved, nil
-			}
-		}
-	}
-	return 0, fmt.Errorf("not find Monitor %s's Brightness", key)
-}
-
 // 判断休眠、待机、屏保、锁屏、关闭显示器等任务能否加入任务队列
 // 优先级为：休眠 > 待机=屏保 > 锁屏=关闭显示器
 func canAddToTasks(sType string, delay int32, tasks metaTasks) bool {
@@ -971,3 +1038,88 @@ func (psp *powerSavePlan) ConnectIdle() error {
 
 	return nil
 }
+
+func (psp *powerSavePlan) initDsgConfig() error {
+	logger.Info("initDsgConfig.")
+	systemBus, err := dbus.SystemBus()
+	if err != nil {
+		return err
+	}
+	psp.systemSigLoop = dbusutil.NewSignalLoop(systemBus, 10)
+	// dsg 配置
+	ds := ConfigManager.NewConfigManager(psp.systemSigLoop.Conn())
+	dsPowerPath, err := ds.AcquireManager(0, dsettingsAppID, dsettingsPowerName, "")
+	if err != nil {
+		return err
+	}
+	dsPower, err := ConfigManager.NewManager(psp.systemSigLoop.Conn(), dsPowerPath)
+	if err != nil {
+		return err
+	}
+	psp.dsgPower = dsPower
+
+	getAllowScreenSaver := func() {
+		data, err := dsPower.Value(0, dsettingsAllowScreenSaver)
+		if err != nil {
+			logger.Warning(err)
+			return
+		}
+		psp.allowScreenSaver = data.Value().(bool)
+		logger.Info("allow screen saver enabled : ", psp.allowScreenSaver)
+	}
+
+	getAllowScreenSaver()
+
+	getDelayWakeupInterval := func() {
+		v, err := dsPower.Value(0, dsettingsDelayWakeupInterval)
+		if err != nil {
+			logger.Warning(err)
+			return
+		}
+		switch vv := v.Value().(type) {
+		case float64:
+			psp.manager.delayWakeupInterval = uint32(vv)
+		case int64:
+			psp.manager.delayWakeupInterval = uint32(vv)
+		default:
+			logger.Warning("type is wrong!")
+		}
+		logger.Info("delay wake up interval : ", psp.manager.delayWakeupInterval)
+	}
+
+	getDelayWakeupInterval()
+
+	getDelayHandleIdleOffIntervalWhenScreenBlack := func() {
+		v, err := dsPower.Value(0, dsettingsDelayHandleIdleOffIntervalWhenScreenBlack)
+		if err != nil {
+			logger.Warning(err)
+			return
+		}
+		switch vv := v.Value().(type) {
+		case float64:
+			psp.manager.delayHandleIdleOffIntervalWhenScreenBlack = uint32(vv)
+		case int64:
+			psp.manager.delayHandleIdleOffIntervalWhenScreenBlack = uint32(vv)
+		default:
+			logger.Warning("type is wrong!")
+		}
+		logger.Info("delay wake up interval : ", psp.manager.delayHandleIdleOffIntervalWhenScreenBlack)
+	}
+	getDelayHandleIdleOffIntervalWhenScreenBlack()
+
+	dsPower.InitSignalExt(psp.systemSigLoop, true)
+	dsPower.ConnectValueChanged(func(key string) {
+		logger.Info("DSG org.deepin.dde.daemon.power valueChanged, key : ", key)
+		switch key {
+		case dsettingsAllowScreenSaver:
+			getAllowScreenSaver()
+		case dsettingsDelayWakeupInterval:
+			getDelayWakeupInterval()
+		case dsettingsDelayHandleIdleOffIntervalWhenScreenBlack:
+			getDelayHandleIdleOffIntervalWhenScreenBlack()
+		default:
+		}
+	})
+
+	return nil
+}
