@@ -51,6 +51,8 @@ type SystemInfo struct {
 	CurrentSpeed uint64
 	// Cpu Hardware
 	CPUHardware string
+	// Os Build
+	OsBuild string
 }
 
 type Daemon struct {
@@ -211,6 +213,11 @@ func (info *SystemInfo) init() {
 		logger.Warning("Get memory capacity failed:", err)
 	}
 
+	info.OsBuild, err = getOsBuild()
+	if err != nil {
+		logger.Warning("Get os build failed:", err)
+	}
+
 	if systemBit() == "64" {
 		info.SystemType = 64
 	} else {
